angular.module("umbraco").component("perplexAiAssistant", {
    templateUrl: "/App_Plugins/Perplex.AI.Assistant/assistant/perplex.ai-assistant.html",
    require: {
        umbProperty: "^^umbProperty",
        umbVariantContent: "^^umbVariantContent"
    },
    controller: [
        "perplexAIMetadataService",
        "$element",
        "$q",
        function perplexAiAssistantController(service, $element, $q) {
            this.MODE = {
                NONE: 0,
                META_TITLE: 1,
                META_DESCRIPTION: 2,
            };

            this.mode = this.MODE.NONE;

            this.isGenerating = false;
            this.generatedMetadata = null; // { metaTitles: ["..."], metaDescription: ["..."] }

            this.$onInit = function $onInit() {
                if (this.umbProperty == null || this.umbProperty.property == null || this.umbVariantContent == null) {
                    this.remove();
                    return;
                }

                service.getConfig()
                    .then(this.determineMode.bind(this))
                    .then(function (mode) {
                        if (mode === this.MODE.NONE) {
                            // If we are not in a specific mode we have no job so remove ourselves.
                            this.remove();
                        } else {
                            // Place us in the correct position in the DOM.
                            this.updatePositionInDom();
                        }
                    }.bind(this));
            }

            this.updatePositionInDom = function updatePositionInDom() {
                var $controls = $element.prev(".controls");
                var $propertyEditor = $controls.find("ng-form *[ng-model]");
                if ($propertyEditor.length > 0) {
                    $element.insertAfter($propertyEditor);
                }
            }

            this.remove = function remove() {
                // Remove ourselves from the DOM
                $element.remove();
            }

            this.determineMode = function determineMode(config) {
                switch (this.umbProperty.property.alias) {
                    case config.metaTitlePropertyAlias:
                        this.mode = this.MODE.META_TITLE;
                        break;

                    case config.metaDescriptionPropertyAlias:
                        this.mode = this.MODE.META_DESCRIPTION;
                        break;

                    default:
                        this.mode = this.MODE.NONE;
                        break;
                }

                // Act as a Promise
                return $q.when(this.mode);
            }

            this.generateMetadata = function generateMetadata(forceRefresh) {
                var nodeId = this.umbProperty.node.id;
                var language = this.umbVariantContent.editor.content.language;
                var culture = language == null ? "" : language.culture;

                this.isGenerating = true;

                if (forceRefresh) {
                    this.generatedMetadata = null;
                }

                return service.generate(nodeId, culture, !!forceRefresh).then(function (metadata) {
                    if (metadata != null) {
                        this.generatedMetadata = metadata;
                    }
                }.bind(this)).finally(function () {
                    this.isGenerating = false;
                }.bind(this));
            }

            this.useMetadata = function useMetadata(value) {
                this.umbProperty.property.value = value;
                this.generatedMetadata = null;
            }
        }
    ]
});
