angular.module("umbraco").service("perplexAIMetadataService", [
    "$http",
    "perplexAIHttpErrorHandler",
    function perplexAIMetadataService($http, errorHandler) {
        var API_ROOT = window.Umbraco.Sys.ServerVariables.umbracoSettings.umbracoPath + "/backoffice/perplexai/metadata/";

        var generationCache = {}; // "<NODE_ID>_<CULTURE>" -> Promise<{ metaTitles: ["..."], metaDescriptions: ["..."] }>

        function save(nodeId, culture, metaTitle, metaDescription) {
            var payload = {
                nodeId: nodeId,
                culture: culture,
                metaTitle: metaTitle,
                metaDescription: metaDescription,
            };

            return $http.post(API_ROOT + "save", payload).then(function (response) {
                return response.data;
            }, errorHandler.handleError);
        }

        function generate(nodeId, culture, forceRefresh) {
            var cacheKey = nodeId + "_" + culture;

            if (forceRefresh) {
                delete generationCache[cacheKey];
            }

            if (generationCache[cacheKey] == null) {
                generationCache[cacheKey] = $http.post(API_ROOT + "generate?nodeId=" + nodeId + "&culture=" + culture);
            }

            return generationCache[cacheKey].then(function (response) {
                if (response != null && response.data != null) {
                    return response.data;
                } else {
                    // Do not cache null values
                    delete generationCache[cacheKey];

                    return null;
                }
            }, function (err) {
                // Do not cache failed Promises
                delete generationCache[cacheKey];

                return errorHandler.handleError(err);
            });
        }

        function getConfig() {
            return $http.get(API_ROOT + "getconfig").then(function (response) {
                return response.data;
            });
        }

        this.save = save;
        this.generate = generate;
        this.getConfig = getConfig;
    }
]);
