angular.module("umbraco").component("perplexAiMetadataTreeNode", {
    templateUrl: "/App_Plugins/Perplex.AI.Assistant/metadata/tree/perplex.ai-metadata-tree-node.html",
    bindings: {
        node: "<",
        save: "&",
        generate: "&",
    },
    controller: ["perplexAIMetadataTreeService", function perplexAiMetadataTreeNodeController(service) {
        this.children = null;
        this.expanded = false;
        this.isGeneratingTitle = false;
        this.isGeneratingDescription = false;
        this.isSaving = false;

        this.generatedMetaTitles = null; // Array of string
        this.generatedMetaDescriptions = null; // Array of string

        this.loadChildren = function loadChildren() {
            return service.getChildren(this.node.id, this.node.culture).then(function (children) {
                this.children = children;
            }.bind(this));
        }

        this.expand = function expand() {
            if (this.children == null) {
                this.loadChildren().then(function () {
                    this.expanded = true;
                }.bind(this));
            } else {
                this.expanded = true;
            }
        }

        this.collapse = function collapse() {
            this.expanded = false;
        }

        this.saveMetadata = function saveMetadata() {
            this.isSaving = true;
            this.save({ node: this.node }).finally(function () {
                this.isSaving = false;
            }.bind(this));
        }

        this.generateMetaTitle = function generateMetaTitle(forceRefresh) {
            this.isGeneratingTitle = true;

            if (forceRefresh) {
                this.generatedMetaTitles = null;
            }

            this.generate({ node: this.node, forceRefresh: forceRefresh }).then(function (metadata) {
                if (metadata != null) {
                    this.generatedMetaTitles = metadata.metaTitles;
                }
            }.bind(this)).finally(function () {
                this.isGeneratingTitle = false;
            }.bind(this));
        }

        this.generateMetaDescription = function generateMetaDescription(forceRefresh) {
            this.isGeneratingDescription = true;

            if (forceRefresh) {
                this.generatedMetaDescriptions = null;
            }

            this.generate({ node: this.node, forceRefresh: forceRefresh }).then(function (metadata) {
                if (metadata != null) {
                    this.generatedMetaDescriptions = metadata.metaDescriptions;
                }
            }.bind(this)).finally(function () {
                this.isGeneratingDescription = false;
            }.bind(this));
        }     

        this.selectMetaTitle = function selectMetaTitle(metaTitle) {
            this.node.metaTitle = metaTitle;
            this.generatedMetaTitles = null;
        }

        this.selectMetaDescription = function selectMetaDescription(metaDescription) {
            this.node.metaDescription = metaDescription;
            this.generatedMetaDescriptions = null;
        }
    }]
});
